/*
  IC22S diode matrix board replcacement giving frequency control and CCTSS generation.
  Denys VK3ZYZ
*/

#define DEBUG     // comment out for final.

#define Version "IC22S_20240401"
#include <EEPROM.h>
#include "Adafruit_GFX.h"
#include "Adafruit_SSD1306.h"
#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 32 // OLED display height, in pixels

// Declaration for an SSD1306 display connected to I2C (SDA, SCL pins)
#define OLED_RESET     -1 // Reset pin # (or -1 if sharing Arduino reset pin)
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);

// define button values. These may need to be changed to suit.
#define Left  7         // ADC Value /25
#define Right 9         // ADC Value /25
#define Both 10         // ADC Value /25
#define ADC_Divide 25   // ADC divider


// synthesiser default settings on start up.
unsigned long RXgenfrequency  = 146050000;
unsigned long TXgenfrequency  = 146050000;
unsigned long IFoffset        =  10700000;  // IF offset
unsigned long PLLoffset       = 131700000;  //133690000;    //check these frequs!
unsigned long chan_spacing    =     25000;  //15000;

float         DisplayFreq     = 146.000;
float         txDisplayFreq   = 146.000;
float         rxDisplayFreq   = 146.000;
int           mode            = HIGH;        // HIGH = normal, LOW = REV

#ifdef DEBUG
  #define       RX                LOW        //normally HIGH. But LOW for debug with no +12V
#else  
  #define       RX                HIGH       //normally HIGH. But LOW for debug with no +12V
#endif

int           TRXmode         = RX;          // HIGH = RX, LOW = TX
int           channel         = 2;           // start on channel number "3" (starts from 0)
int encoder;                                 // encoder direction: CCW=-1; NC=0; CW=1
int timer                     = 100;         // sub tone setting. The higher the number, the slower the timing.
int i ;
unsigned long temp;
int temp2;
int temp3;
int loopcount;
int validButton;
int Function;
int oldButton;
int FunctionDone;

// this set has 40 channels
int EEPROM_RXFrequs           =   0;          // address in EEPROM of RXFrequs table. 40x4 =   0 to 159
int EEPROM_TXFrequs           = 160;          // address in EEPROM of TXFrequs table  40x2 = 160 to 239
int EEPROM_SubTone            = 240;          // address in EEPROM of SubTone  table  40x2 = 240 to 319
int EEPROM_Flag               = 320;          // address in EEPROM of 'written' flag
int EditFlag                  =   0;          // 0 = normal, 1 = freq, 2 = ctcss, 3 = offset.
int DoEEPROM                  = 590;
int EditNext                  = 500;
int Confirm                   = 300;

// encoder definitions
#define ENCODER_A     A0
#define ENCODER_B     A1
#define TRX           A2           // PC2 input HIGH = TX, LOW = RX.
#define Encoder_Switch    A3           // PC3 Encoder Button  repeater reverse
// A4 = SDA
// A5 = SCL
#define ModeSwitch    A6           // DupA >=300, spx = 0, DupB >=700 (461 and 839)
//                    A7           // not used yet

// Use Port D to drive the PLL pins.
// NOTE the Arduino 0 and 1 pins swap the port bit order.
//                     1           // PLL b0 PD0 (Serial TXD)
//                     0           // PLL b1 PD1 (Serial RXD)
//                     2           // PLL b2 PD2 
//                     3           // PLL b3 PD3 
//                     4           // PLL b4 PD4 
//                     5           // PLL b5 PD5 
//                     6           // PLL b6 PD6 
//                     7           // PLL b7 PD7 

/* ********************* PortB0-PB4 used to generate CTCSS tones via R2R nerwork.
#define spare2         8           // Sub Tone R2R b0 PB0
#define spare3         9           // Sub Tone R2R b1 PB1
#define spare4        10           // Sub Tone R2R b2 PB2
#define spare5        11           // Sub Tone R2R b3 PB3
#define spare6        12           // Sub Tone R2R b4 PB4
*/

#define Output2       13           // not used yet

#define ENCODER_INPUT_PORT PINC
#define PLL_Port PIND
#define On  1
#define Off 0
#define RefFrequ      500000       // 16Mhz / 32 steps for CTCSS sine wave generator
                                    
int     SineDivider = 4065;         // default sine wave divider
  
long int RXFrequs []      // Edit this for RX frequencies wanted.
{ 146000,
  146625,
  146650,
  146675,
  146700,
  146725,
  146750,
  146775,
  146800,
  146825,
  146850,
  146875,
  146900,
  146925,
  146950,
  146975,
  147000,
  146425,
  146450,
  146475,
  146500,
  146525,
  146550,
  146575,
  146600,
  147025,
  147050,
  147075,
  147100,
  147125,
  147150,
  147175,
  147200,
  147225,
  147250,
  147275,
  147300,
  147325,
  147350,
  147375,
};

int TXFrequs []      // Edit this for TX frequencies wanted. Added to RX frequs.
{ 0,
  -600,
  -600,
  -600,
  -600,
  -600,
  -600,
  -600,
  -600,
  -600,
  -600,
  -600,
  -600,
  -600,
  -600,
  -600,
  -600,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  600,
  600,
  600,
  600,
  600,
  600,
  600,
  600,
  600,
  600,
  600,
  600,
  600,
  600,
  600,
};

// Changing SubTone to int lets more channels be defined.
int SubTone []      // Edit this for CTCSS frequencies wanted.(freq x 10)
{ 0,      // "0 ="  chan 1
  0,
  19,  //  chan 3
  0,
  10,   //  chan 5
  0,
  10,   //  chan 7
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  10,   // chan 15
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  19,    // chan 26
  0,
  0,
  10,   // chan 29
  10,   // chan 30
  0,
  0,
  0,
  0,
  0,
  10,   // chan 36
  0,
  0,
  0,
  10,   // chan 40
};

float CTCSS = 0;
int   CTCSS_Index = 0;
int   CTCSS_Tones []
//   0     1     2     3     4     5     6     7     8     9
{    0,  670,  693,  719,  744,  770,  797,  825,  854,  885,   //  +0
   915,  948,  974, 1000, 1035, 1072, 1109, 1148, 1188, 1230,   // +10
  1273, 1318, 1365, 1413, 1462, 1500, 1514, 1567, 1598, 1622,   // +20
  1655, 1679, 1713, 1773, 1799, 1835, 1862, 1899, 1928, 1966,   // +30
  1995, 2035, 2065, 2107, 2181, 2257, 2291, 2336, 2418, 2503,   // +40
  2541,                                                         // +50
};

int sinetable[] // one cycle sine wave via R2R network.
{ 0,1,2,3,5,7,10,13,16,19,22,25,27,28,29,30,31,30,29,28,27,25,22,19,16,13,10,7,5,3,2,1, };


void setup() {

  if(!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) { // Address may be 0x3D 
    for(;;); // Don't proceed, loop forever
  }
  display.clearDisplay();
  display.display();                  // Update screen
  display.setTextSize(1);             // Normal 1:1 pixel scale
  display.setTextColor(WHITE,BLACK);  // Draw white text
  display.setCursor(0,0);             // Start at top-left corner
  display.println(F(Version));        // This tells you what version the code is
  display.display();                  // Update screen
  delay(2000);                        // wait 2 seconds
  
  // configure rotary encoder & push-button
  pinMode(ENCODER_A, INPUT_PULLUP);   // encoder leads
  pinMode(ENCODER_B, INPUT_PULLUP);
  pinMode(Encoder_Switch, INPUT_PULLUP);  // Hi = nomal, LOW =  Reverse
  pinMode(TRX, INPUT);                // Hi = TX, LOW =  RX
  DDRB  = B11111;                     // sets Arduino pins 8 to 12 as outputs.
  PORTB = B10000;                     // R2R out at rest to half level.
  DDRD  = B11111111;                  // all bits outputs

  encoder = read_encoder();           // returns +1/0/-1 (cw/nc/ccw)
  mode=( digitalRead(Encoder_Switch));    // read FWR/REV mode
  TRXmode=(digitalRead(TRX));         // and TX/RX mode
  set_synth(mode, TRXmode);           // set the PLL accordingly
  loopcount = 0;
  
#ifdef DEBUG
  Serial.begin(115200);
 #endif

temp2 =  analogRead(ModeSwitch);      // Is a button down?

 if (temp2 >=100)                     // button down on power up/
 { display.clearDisplay();
   BigChars();
   display.print("Defaults");
   display.display();  
   while (temp2 != 0)
    {temp2 =  analogRead(ModeSwitch);      // Is a button down? 
    }
  }
  
  else
  { display.clearDisplay();
    BigChars();
    display.print("EEPROM");
    display.display();
    delay(1000);
    read_RXFrequs();
    read_TXFrequs();
    read_SubTone(); 
  }
  
  NormalDisplay();
  validButton =LOW;   // no button pressed
  EditFlag =0;  
  FunctionDone =0;
}

/******************************
  main loop
*******************************/
void loop() {
  
loopcount ++;
if (loopcount >= 200)                     // read buttons every 200 loops
  { loopcount = 0;  
    temp2     = 0;      
  while(temp2 !=  analogRead(ModeSwitch)) // read the buttons
    {   temp2  =  analogRead(ModeSwitch); // loop until 2 reading the same
    }
                                                 
if (temp2 == 0)                           // no button pressed!
 {  validButton = LOW; 
    Function    =   0;
    FunctionDone=   0;                    // allow a new button
 }
 else if (oldButton != temp2)             // Button, but not the same reading
      {  oldButton   = temp2;             // save it for next time
      }
      else
      { validButton = HIGH;              // Yes, valid button pressed!
      }

 if (validButton ==  HIGH)
 {
    Function = (temp2/ADC_Divide);        // convert to single digit
    Function = (Function + FunctionDone); // 
 }
 
                                         
#ifdef DEBUG
Serial.print("Temp2=");
Serial.print(temp2);
Serial.print(" Function=");
Serial.println(Function);
#endif


if  (FunctionDone==0)                // stop a repeat 
{
    switch (Function)
    { case Right:                       // right button
      display.clearDisplay();
      BigChars();
      display.print("Write?");
      Yes();
      if (digitalRead(Encoder_Switch) == LOW) // do this until encoder switch pushed
       { while (digitalRead(Encoder_Switch) == LOW) // wait until encoder switch released
          {
          }
        display.clearDisplay();
        BigChars();
        display.print("SAVING");
        write_RXFrequs();
        write_TXFrequs();
        write_SubTone();
        EditFlag =0;  
        FunctionDone=10;                // stop a repeat  
        NormalDisplay();
       } 
      break;

      case Left:                               // left button
        EditFlag ++; 
        if (EditFlag ==4)
          { EditFlag =0;
            NormalDisplay();
        FunctionDone=10;                // stop a repeat  
        NormalDisplay();
          }
      break;

      case 6:                               // both buttons
        {     
        } 
      break;
    }
 
     if (EditFlag == 1)
      { EditMessage();
        if (digitalRead(Encoder_Switch) == LOW) // do this until encoder switch pushed
          { EncoderButtonUp();
            AdjustRXFrequs();
        FunctionDone=10;                // stop a repeat  
        NormalDisplay();
          }
    }
  
     if (EditFlag == 2)
      { EditMessage();
        if (digitalRead(Encoder_Switch) == LOW) // do this until encoder switch pushed
          { EncoderButtonUp();
            AdjustCTCSS();
        FunctionDone=10;                // stop a repeat  
        NormalDisplay();
      }
     }

     if (EditFlag == 3)
      { EditMessage();
        if (digitalRead(Encoder_Switch) == LOW) // do this until encoder switch pushed
          { EncoderButtonUp();
            AdjustTXFrequs();
        FunctionDone=10;                // stop a repeat  
        NormalDisplay();
        }
     }
 }  
}

  else                                            // not button time, just run as normal
  { if ( digitalRead(Encoder_Switch) != mode)         // has the Reverse switch changed?
    { mode=( digitalRead(Encoder_Switch));            // Yes, set the mode to suit.
    set_synth(mode, TRXmode);                     // set the PLL
    }
  if (digitalRead(TRX) !=TRXmode)                 // has the TX/RX mode changed?
    { TRXmode=(digitalRead(TRX));                 // Yes, set the mode to suit.
      set_synth(mode, TRXmode);                   // set the PLL
    }
  if (TRXmode==RX)                                // check for frequency chanegs if RX only.
    { encoder = read_encoder();                   // returns +1/0/-1 (cw/nc/ccw)
      if (encoder != 0)                           // has the encoder been moved?
        { channel = channel + encoder;            // Yes, inc/dec channel number
          channel = constrain(channel, 0, 39);    // max 40 channels.
          set_synth(mode, TRXmode);               // set the PLL
        }
    }
  }
}



/******************************
  read encoder direction
*******************************/
int read_encoder() {
  // definitions
#define DELAY 100                                 // number of polling-loops   
  static unsigned short integrator;               // encoder debounce

  /* the array values are determined by combining the previous and current A1,A0
    Gray Code bit-patterns to form an array-index. +1 is assigned to all clockwise
    (CW) indexes; -1 is assigned to all counter-clockwise (CCW) indexes; and 0 is
    assigned to the remaining positions to indicate no movement*/
  static char encoder[] = {0, -1, 1, 0, 1, 0, 0, -1, -1, 0, 0, 1, 0, 1, -1, 0}; // SWAPPED PHASES
  static unsigned char encoder_index = 0;
  static long count = 0;                         // CW rotation = +1; CCW rotation = -1
  static long this_reading;
  static long last_reading;
  char value;                                    // return value

  /* check for encoder rotation. The absolute "count" value changes as the encoder rotates.
    Typically there are four states (counts) per click, but often more due to contact bounce.
    Bounce doesn't matter so long as the "count" decreases" for CCW rotation; remains constant
    when there is no rotation; and increases with CW rotation. Bounce can be completely
    eliminated if we look for the "detent" code and integrate as shown below. */
  encoder_index <<= 2;                           // shift previous bit-pattern left two places
  encoder_index |= (ENCODER_INPUT_PORT & 0x03);  // get current bit-pattern and combine with previous
  count += encoder[( encoder_index & 0x0f )];

  // debounce encoder
  if ((ENCODER_INPUT_PORT & 0x03) == 3) {        // encoder on an indent (3 decimal == 00000011 binary)
    if (integrator < DELAY) {
      integrator++;
    }
  } else {                                   // encoder not resting on indent
    if (integrator > 0) {
      integrator--;
    }
  }

  // prepare the "return" value
  if (integrator >= DELAY) {                 // encoder deemed to be stationary
    this_reading = count;
    if (this_reading < last_reading) value = -1;
    if (this_reading == last_reading) value = 0;
    if (this_reading > last_reading) value = 1;
    last_reading = this_reading;             // both readings now hold current "count"
    integrator = 0;                          // reset integrator
  } else {
    value = 0;
  }
  return (value);
}

void set_synth( int frmode, int trx )       // set the display and PLL
{ ShowChannel();
  ShowTX();
  ShowCTCSS();                             // display the CTCSS status
                                           // Select FWD or REV frequencies
  if (frmode == HIGH)                      // test for fwd or rev repeater...
  { RXgenfrequency = (RXFrequs[channel]);  // normal operation
    TXgenfrequency = ((RXFrequs[channel]) + (TXFrequs[channel]));
  }
  else
  { TXgenfrequency = (RXFrequs[channel]); // reverse repeater operation
    RXgenfrequency = ((RXFrequs[channel]) + (TXFrequs[channel]));
  }                                       // Then display them....
  if ( trx == RX)                         // is it = RX?
    { DisplayFreq = RXgenfrequency;       // Yes, display the RX frequency
      ShowFreq();
    }
   else                                   // No, it is TX.
    { display.setTextColor(BLACK,WHITE);  // Invert Frequ Display
      DisplayFreq = TXgenfrequency;       // display the TX frequency
      ShowFreq();
    }
  display.setTextColor(WHITE,BLACK);      // back to normal text
  display.display();                      // Update screen

  if ( trx == RX)                         // is it transmittig or receiving?          
  { setPLL(RXgenfrequency);               // =RX.
    TIMSK1 = 0;                           // subtone off. Not needed for RX.
  }
  else                         
  { setPLL(TXgenfrequency);               // =TX.
    if (CTCSS_Index != 0)                 // do we need subtones?   
      { SineDivider = (RefFrequ/CTCSS);   // Yes, set the timer
        CTCSS_Tone();                     // and turn subtone on
      }
    else 
      { TIMSK1 = 0;                       // subtone off
      }
  }
}

void CTCSS_Tone()                         // CTCSS tone generation setup
{ cli();                                  // interupsts off for setup
  TCCR1A = 0;                             // 
  TCCR1B = 0;                             //  no prescaler
                                          // set compare match register for 1hz increments
  OCR1A = SineDivider ;                   // = (16*10^6) / (1*1024) - 1 (must be <65536)
  TCCR1B |= (1 << WGM12);                 // turn on CTC mode
  TCCR1B |= (0 << CS12) | (0 << CS11) | (1 << CS10);    // Set CS10 no prescaler
  TIMSK1 |= (1 << OCIE1A);                // enable timer compare interrupt
  sei();                                  // interupsts on      
}

ISR (TIMER1_COMPA_vect)                   // generate the CTCSS tone
   { PORTB=(sinetable[i++]);              // write value from table to PORTB
     if(i>=32)                            // 32 step loop for next value
      { i=0;
      }
  }

// make this the drive for the diode matrix replacement. 25Khz channels for Australia
void setPLL(unsigned long frq)            // program PLL
{ int PLLdivider = ((frq - 144400)/25);   // convert frequency to PLL number
  PORTD=(PLLdivider);                     // write to PLL.
}


void write_RXFrequs ()
{ writeLongArrayIntoEEPROM( EEPROM_RXFrequs, RXFrequs, 40);
}

void write_TXFrequs ()
{ writeIntArrayIntoEEPROM( EEPROM_TXFrequs, TXFrequs, 40);
}

void write_SubTone ()
{ writeIntArrayIntoEEPROM( EEPROM_SubTone, SubTone, 40);
}

void read_RXFrequs ()
{ readLongArrayFromEEPROM( EEPROM_RXFrequs, RXFrequs, 40);
}

void read_TXFrequs ()
{ readIntArrayFromEEPROM( EEPROM_TXFrequs, TXFrequs, 40);
}

void read_SubTone ()
{ readIntArrayFromEEPROM( EEPROM_SubTone, SubTone, 40);
}
// int numbers.....
void writeIntArrayIntoEEPROM(int address, int numbers[], int arraySize)
{
  int addressIndex = address;
  for (int i = 0; i < arraySize; i++) 
  {
    EEPROM.write(addressIndex, numbers[i] >> 8);
    EEPROM.write(addressIndex + 1, numbers[i] & 0xFF);
    addressIndex += 2;
  }
}

void readIntArrayFromEEPROM(int address, int numbers[], int arraySize)
{
  int addressIndex = address;
  for (int i = 0; i < arraySize; i++)
  {
    numbers[i] = (EEPROM.read(addressIndex) << 8) + EEPROM.read(addressIndex + 1);
    addressIndex += 2;
  }
}

// long numbers......
void writeLongArrayIntoEEPROM(int address, long numbers[], int arraySize)
{
  int addressIndex = address;
  for (int i = 0; i < arraySize; i++)
  {
    EEPROM.write(addressIndex, (numbers[i] >> 24) & 0xFF);
    EEPROM.write(addressIndex + 1, (numbers[i] >> 16) & 0xFF);
    EEPROM.write(addressIndex + 2, (numbers[i] >> 8) & 0xFF);
    EEPROM.write(addressIndex + 3, numbers[i] & 0xFF);
    addressIndex += 4;
  }
}
void readLongArrayFromEEPROM(int address, long numbers[], int arraySize)
{
  int addressIndex = address;
  for (int i = 0; i < arraySize; i++)
  {
    numbers[i] = ((long)EEPROM.read(addressIndex) << 24) +
                 ((long)EEPROM.read(addressIndex + 1) << 16) +
                 ((long)EEPROM.read(addressIndex + 2) << 8) +
                 (long)EEPROM.read(addressIndex + 3);
    addressIndex += 4;
  }
}

// Display Stuff.......

void ShowCTCSS()                          // show the CTCSS frequency..
{ display.setTextSize(1);                 // Normal 1:1 pixel scale
  display.setCursor(90,0);                // set cursor
  ShowCTCSSfrq();
}

void ShowCTCSSfrq()
{   CTCSS_Index=(SubTone[channel]);       // get the CTCSS number
  if(CTCSS_Index!=0)                      // if not zero,
    { CTCSS = ((CTCSS_Tones[CTCSS_Index])/10);  // then get the actual CTCSS freq in hz
      if(CTCSS<100)                       // is it less than 100hz?
        { display.print(" ");             // pad leading space if needed
        }  
      display.print(CTCSS,1);             // display the CTCSS frequ to 1 decimal place
      display.print(" ");
    }
    else { display.print("notone");    // no CTCSS for this channel, so clear the CTCSS display.
         }
  display.display();                      // update the display.
}

void AdjustCTCSS()                        // modify the CTCSS number index for this channel.
  {   BigCTCSS();                         // show current CTCSS
while (digitalRead(Encoder_Switch) == HIGH) // do this until encoder swicth pushed
  { encoder = read_encoder();             // look at the encoder
     if (encoder != 0)                    // Has the encoder moved?
      { temp3 = (SubTone[channel]);       // Yes, get the current CTCSS number (0-50)
        temp3 = temp3 + encoder;          // add in te encoder
        temp3 = constrain(temp3, 0, 50);  // limit the number to the valid range       
        (SubTone[channel]) = temp3;       // put the new number back in te table
        BigCTCSS();
      }                                   // go back to do it again if mode still valid
  }    
 NormalDisplay();
}

void BigCTCSS()
{ BigChars();       
  ShowCTCSSfrq();                   // then show the new CTCSS. 
}

void ShowFreq()                           // display the current frequency for this channel
{ BigChars();
  DisplayFreq = (DisplayFreq)/1000;       // convert the freq (in khz) to Mhz
  display.print(DisplayFreq, 3);          // display the frequ fo 3 decimal places.
  display.display();                      // Update screen
}

void ShowFreqAdj()                        // display the frequ for channel being edited
{ BigChars(); 
  display.print("14");                    // display the "14" part of the 14xxxx
  DisplayFreq = (DisplayFreq - 140000);   // remove the "14" part of the frequency
  DisplayFreq = (DisplayFreq)/1000;       // convert remainder to Mhz
  display.setTextColor(BLACK,WHITE);      // invert text
  display.print(DisplayFreq, 3);          // display the frequ fo 3 decimal places.
  display.display();                      // Update screen
  display.setTextColor(WHITE,BLACK);      // back to normal text
}

void AdjustRXFrequs()                      // Adjust the channel frequency
{    //Yes(); 
while (digitalRead(Encoder_Switch) == HIGH) // do this until encoder switch pushed
  {
      encoder = read_encoder();           // look at the encoder
     if (encoder != 0)                    // has it moved?
      { temp = (RXFrequs[channel]);       // yes, get the current frequency (in khz)
        temp = temp + (encoder*25);       // add the encoder to it. (in 25Khz steps)
        temp = constrain(temp, 145000, 147975); // limit the frequency to valid band
        (RXFrequs[channel]) = temp;       // update the channel frequency
        DisplayFreq = (RXFrequs[channel]);// get the current channel frequency
        ShowFreqAdj();                   // display the frequency
      } 
                                   // go back to do it again if mode still valid 
  }
 NormalDisplay();                                    
}

void AdjustTXFrequs()
{   //Yes();
    delay(100);
    int xxx;
    ShowTXAdj();
while (digitalRead(Encoder_Switch) == HIGH) // do this until encoder switch pushed
  {
    { xxx = (TXFrequs[channel]);   // get the current channel frequency
      encoder = read_encoder();           // look at the encoder
     if (encoder != 0)                    // has it moved?
{
       xxx = xxx + (encoder*600);      // add the encoder to it.
       xxx = constrain(xxx,-600,600);// limit to valid value
        (TXFrequs[channel])= xxx;
        ShowTXAdj();
      } 
    } 
  }                                       // go back to do it again if mode still valid 
 NormalDisplay();
}

void ShowTXAdj()
{ Yes(); 
  BigChars();
  ShowTXoffset(); 
}



void ShowTX()
{ display.setTextSize(1);             // Normal 1:1 pixel scale
  display.setCursor(50,0);                  // write mode to the LCD
  ShowTXoffset();
}


void ShowTXoffset()
{ if (TXFrequs[channel]== 0)
   {display.print(" SIM "); }
  if (TXFrequs[channel] == 600)
   {display.print("+600"); }
  if (TXFrequs[channel] ==-600)
   {display.print("-600"); }
  display.display();  
}

void ShowChannel()
{ display.setTextSize(1);                   // Normal 1:1 pixel scale
  display.setCursor(0,0);                  // write channel number to the LCD 60
  display.print("Ch   ");                      // clear the old number
  display.setCursor(18,0);                  // write channel number to the LCD
  if (channel < 9)                          // "9" = 10 as array starts at 0
  { display.print(" ");                     // extra space for formalling a single digit.
  }
  display.print((channel + 1));             // display the channel number
}

void EditMessage()
{ display.clearDisplay();
  display.display();                  // Update screen
  display.setTextSize(1);             // Normal 1:1 pixel scale
  display.setTextColor(WHITE,BLACK);  // Draw white text
  display.setCursor(0,10);             // Start at top-left corner
if (EditFlag ==1)
  {display.println("edit RX freqs?");   // the message to print
  }
if (EditFlag ==2)
  {display.println("edit CTCSS freqs?");// the message to print
  }
if (EditFlag ==3)
  {display.println("edit TX freqs?");   // the message to print
  }
  Yes();
}

void NormalDisplay()
{ display.clearDisplay();
  display.display();                  // Update screen
  ShowChannel();
  ShowCTCSS();                        // show the final CTCSS in normal text, not inverted.
  ShowTX();
  DisplayFreq = (RXFrequs[channel]); 
  ShowFreq(); 
  EditFlag=0; 
}

void Yes()
{ display.setTextSize(1);             // Normal 1:1 pixel scale
  display.setCursor(0,0);             // Start at next line
  display.println("Push encoder = YES");    
  display.display();                  // Update screen
}

void ButtonUp()
{  temp2        = 100; 
 while(temp2 !=   0)                     // wait for button up...
   {   temp2  =  analogRead(ModeSwitch); //  
   }
}

void EncoderButtonUp()
{while (digitalRead(Encoder_Switch) == LOW) // wait until encoder switch released
   {
   }
}  

void BigChars()
{ display.setTextSize(3);             // big chars
  display.setCursor(1,10);            // write mode to the LCD 
  display.print("       ");
  display.display();
  display.setCursor(1,10);            // write mode to the LCD 
  
}
